// @author kelvin

#include <iostream>
#include <vector>
#include <string>
#include <algorithm>
#include "poly.h"

using namespace std;

poly::poly(comp co0) {
   co.resize(1);
   co[0]=co0;
}
poly::poly(vector<comp> _co):co(_co) {
   trim();
}
poly::poly(vector<string> str) {
   co.resize(str.size());
   for(int i=0;i<=degree();i++)
      co[i]=comp(str[i]);
   trim();
}

// helper function
int poly::degree() const {
   return (int)co.size()-1;
}
void poly::trim() {
   while(degree()&&co.back()==comp(.0,.0)) co.pop_back();
}
void poly::show() const {
   for(int i=0;i<=degree();i++) {
      co[i].show();
      cout << (i<degree()?' ':'\n');
   }
}

// operators
poly& poly::operator+=(const poly &b) {
   if(b.degree()>degree()) co.resize(b.degree()+1);
   for(int i=0;i<=degree();i++)
      co[i]+=i<=b.degree()?b.co[i]:comp(.0,.0);
   trim();
   return *this;
}
poly poly::operator+(const poly &b) const {
   return poly(*this)+=b;
}

poly& poly::operator-=(const poly &b) {
   if(b.degree()>degree()) co.resize(b.degree()+1);
   for(int i=0;i<=degree();i++)
      co[i]-=i<=b.degree()?b.co[i]:comp(.0,.0);
   trim();
   return *this;
}
poly poly::operator-(const poly &b) const {
   return poly(*this)-=b;
}

poly& poly::operator*=(const poly &b) {
   *this=*this*b;
   return *this;
}
poly poly::operator*(const poly &b) const {
   vector<comp> cc(degree()+b.degree()+1);
   for(int i=0;i<=degree();i++)
      for(int j=0;j<=b.degree();j++)
         cc[i+j]+=co[i]*b.co[j];
   return poly(cc);
}

poly poly::operator()(const poly &b) const {
   poly c,m=comp(1.0,0.0);
   for(int i=0;i<=degree();i++) {
      c+=m*co[i];
      m*=b;
   }
   return c;
}

poly poly::eval(comp x) const {
   return (*this)(vector<comp>(1,x));
}
