// @author kelvin

#ifndef __POLY_H
#define __POLY_H

#include <iostream>
#include <iomanip>
#include <algorithm>
#include <vector>
#include <cstdlib>
#include <cmath>

using namespace std;

const double eps=1e-9;
inline bool iszero(double x) { return fabs(x)<eps; }
inline bool isequal(double a,double b) { return iszero(a-b); }

class comp {
   public:
      double re,im;
      comp(double _re=.0,double _im=.0):re(_re),im(_im) {}
      comp(string str) { // a, bi, a+bi
         re=im=.0;
         int oi=str.find_first_of("+-",1);
         int ji=str.rfind("i");
         if(oi!=string::npos||ji==string::npos) re=atof(str.c_str());
         if(ji!=string::npos) im=atof(str.substr(oi==string::npos?0:oi).c_str());
      }
      comp& operator+=(const comp &b) { re+=b.re; im+=b.im; return *this; }
      comp operator+(const comp &b) const { return comp(*this)+=b; }
      comp& operator-=(const comp &b) { re-=b.re; im-=b.im; return *this; }
      comp operator-(const comp &b) const { return comp(*this)-=b; }
      comp& operator*=(const comp &b) {
         double tre=re*b.re-im*b.im;
         double tim=re*b.im+im*b.re;
         re=tre;
         im=tim;
         return *this;
      }
      comp operator*(const comp &b) const { return comp(*this)*=b; }
      bool operator==(const comp &b) const { return isequal(re,b.re)&&isequal(im,b.im); }
      void show() const {
         cout << fixed << setprecision(2);
         if(!iszero(re)||iszero(im)) {
            cout << re;
            cout.setf(ios::showpos);
         }
         if(!iszero(im)) cout << im << 'i';
         cout.unsetf(ios::showpos);
      }
};

class poly {
   public:
      vector<comp> co;
      // constructor
      poly(comp co0=comp(.0,.0));
      poly(vector<comp> _co);
      poly(vector<string> str);
      // helper function
      int degree() const;
      void trim();
      void show() const;
      // operators
      poly& operator+=(const poly &b);
      poly operator+(const poly &b) const;
      poly& operator-=(const poly &b);
      poly operator-(const poly &b) const;
      poly& operator*=(const poly &b);
      poly operator*(const poly &b) const;
      poly operator()(const poly &b) const;
      poly eval(comp x) const;
};

#endif
